// Persistence of Vision Ray Tracer Scene Description File
// File: sc2hair.inc
// Vers: 3.6.1
// Desc: Macros to convert Surcoat Data to hair/fur
// Date: 19.01.05 (dd.mm.yy)
// Auth: Tim Nikias Wenclawiak - www.NoLights.de
// Last Update: 28.06.08 (dd.mm.yy)

//Description
// The Sc2Hair Plug-In for the Surcoat Macros displays the samples with hair. There are various
// Macros which can be (and are meant to be) User-Scripted to alter the way the hairs lie, their
// length and their color. Currently, only straight strands are possible, but again, a few macros
// may be declared and adjusted to make use of customized hair macros.
// Note that the main purpose of the macros is to avoid creating a unique object for every single
// hair, as this leads to quite large memory consumption. Instead, the macros create a set of
// hairs which will be "interpolated" to fit the Surcoat-Sample. That is, hairs are generated on
// a unit-sphere and the one closest to the currently needed hair-direction will be used and
// transformed to fit the slightly different angle. I coined these hairs "IHairs", short for
// Interpolation-Hairs. So instead of generating 4000 hairs and requiring volumes of RAM for that,
// I generate only a few hundred and reuse the hairs over and over. Less memory consumption, and
// the side-effect of reusing the same hairs is unnoticeable when you use a good amount of IHairs.

// There are four ways of using hair:

// Non-I/O *complete* (builds a hair per sample!)
// 1. Sc2Hair ( Surcoat_Sample_File, <Base_Rad,Tip_Rad>, <min_Length,max_Length>)

// Non-I/O (Reparse everytime)
// 1. Create_IHairs ( ICube_Length, Length_Res, <Base_Rad,Tip_Rad>, <min_Length,max_Length>)
// 2. Sc2Hair_UseCreated ( Surcoat_Sample_File)

// I/O (IHairs are written to disk and can be used in other images as well)
// 1. Write_IHairs ( ICube_Length, Length_Res, <Base_Rad,Tip_Rad>, <min_Length,max_Length>, IHair_File)
// 2. Sc2Hair_UseSaved ( Surcoat_Sample_File, IHair_File)

// OnTheFly (even less memory and harddrive needed, but the resulting file is only really suitable for the current scene, as
//           some IHairs will be missing unless you need *all* of them in the scene)
// 1. Sc2Hair_OnTheFly ( Surcoat_Sample_File, ICube_Length, Length_Res, <Base_Rad,Tip_Rad>, <min_Length,max_Length>, IHair_File)

// If you just want to have a single hair, use the SingleHair-Macro. It is called like this
// SingleHair(Hairbase, Hairnormal, <Length,Base_Radius,Tip_Radius>)

#ifndef (Surcoat_Debug) #declare Surcoat_Debug = on; #end
#declare Sc2Hair_Rand = seed(260981);
#declare Sc2Hair_IndividualTexture = 0;
#declare Sc2Hair_YSwitch = 0;

//Creating hair:
// A few parameters are needed to define the properties of the hairs:
//Gravity and Wind settings
#declare Hair_Gravity = -y;
#declare Hair_Wind = <0,0,0>;
#declare Hair_WindStrength = .75;
//Detail settings
#declare Hair_FixedSteps = on; //sets if steps along length are controlled as "total" or "per unit"
#declare Hair_Steps = <8,15>; //<along diameter, along length>
#declare Hair_BentPerUnit = 60; //maximum bend due to gravity along one unit of hair, think of it as a "strength" parameter
//Modify the x:y ratio of the hair
#declare Hair_BaseStretchRatio=1;
#declare Hair_TipStretchRatio=1;


//Macros used for arithmetic reasons to circumvent matrix problems
#macro VNormalize(_Vec)
 #if (vlength(_Vec))
  #local _Ret=vnormalize(_Vec); #else
  #local _Ret=<0,0,0>; #end
 _Ret
#end
#macro xVnormalize(_Vector)
  #if (vlength(_Vector)>0)
    #local _Return=vnormalize(_Vector);
  #else
    #local _Return=x;
  #end
  _Return
#end
#macro yVnormalize(_Vector)
  #if (vlength(_Vector)>0)
    #local _Return=vnormalize(_Vector);
  #else
    #local _Return=y;
  #end
  _Return
#end

//************************************  MACROS THAT MAY/OUGHT TO BE CHANGED BY THE USER

//"Modify_IHairSample"
//Function:
// Possibility to tweak position & normal, along
// with defining the length for the hair at this
// sample.
//Parameters:
// 1. Position
// 2. Normal
// 3. Length
// 4. Possible_Lengths (just hands the min/max-lengths for ease of use)
//Returns:
// #declare the given parameters to modify them
//Notes:
// This macro is called from within the display-macro to
// give the User the ability to modify the hair-evaluation
#macro Modify_IHairSample(Position,Normal,Length,Possible_Lengths)
  #declare Position=Position;
  #declare Normal=Normal;
  #declare Length=Possible_Lengths.u;
#end

//"Sc2Hair_Texture"
//Function:
// Textures a single hair
//Parameters:
// 1. Position/Base of hair
// 2. Original-Normal of hairbase
// 3. Modified-Normal of hairbase
// 4. Length of hair
//Returns:
// Some sort of texture-description
//Notes:
// User-Definable
#macro Sc2Hair_Texture(Position,Original_Normal,Modified_Normal,Length)
  #local _Pig = pigment{bozo color_map{[0 rgb 0][.4 rgb 0][.6 rgb 1][1 rgb 1]}}
  #local _F=function{pigment{_Pig}}
  #local _Pos = Position*<1,1,1>;
  #local _Rgb_T = _F(_Pos.x,_Pos.y,_Pos.z);
  #local _Rgb = <_Rgb_T.x,_Rgb_T.y,_Rgb_T.z>;
  texture{pigment{rgb <1,1,1>*_Rgb}}
#end

//"Create_IHair"
//Function:
// Macro that will be called during IHair creation. Given a normal
// and Size-Vector this macro is supposed to return a mesh2-object.
//Parameters:
// 1. Normal
// 2. Size-Vector (<Length, Base_Radius, Tip_Radius>)
//Returns:
// Mesh2-Object
//Notes:
// None
#macro Create_IHair(_Normal,_Size)
  #local _Temp = Create_Hairline(_Normal,_Size.x);
  #local _Return = Create_Hairmesh(_Temp,_Size);
  _Return
#end

//"Write_IHair"
//Function:
// Macro that will be called during IHair Writing. Given a normal
// and Size-Vector this macro is supposed to write a mesh2-object
// to _sc2hair_FileOut
//Parameters:
// 1. Normal
// 2. Size-Vector (<Length, Base_Radius, Tip_Radius>)
//Returns:
// None (but writes mesh2 to _sc2hair_FileOut)
//Notes:
// None
#macro Write_IHair(_Normal,_Size)
  #local _Temp = Create_Hairline(_Normal,_Size.x);
  Write_Hairmesh(_Temp,_Size)
#end

//"OnTheFly_IHair"
//Function:
// Macro that will be called during IHair OnTheFly-Writing. Given a normal
// and Size-Vector this macro is supposed to write a mesh2-object
// to _sc2hair_FileOut *AND* return a mesh2-object!
//Parameters:
// 1. Normal
// 2. Size-Vector (<Length, Base_Radius, Tip_Radius>)
//Returns:
// Returns Mesh2-object as well as writing the mesh2 to _sc2hair_FileOut
//Notes:
// None
#macro OnTheFly_IHair(_Normal,_Size)
  #local _Temp = Create_Hairline(_Normal,_Size.x);
  #local _Return = OnTheFly_Hairmesh(_Temp,_Size);
  _Return
#end

//************************************  MACROS TO CONVERT BETWEEN NORMALS AND ARRAY-INDICES

//"Normal2Index"
//Function:
// Given a normal, returns the array-index with closest normal
//Parameters:
// 1. Normal
// 2. Resolution of IHair
//Returns:
// 2. UV-Vector
//Notes:
// None
#macro Normal2Index(_Normal,HairArray_Res)
  #local _Box = box{0,1}
  #local _Hit = <0,0,0>;
  #local _End = trace(_Box,.5,_Normal,_Hit);
  #while (!vlength(_Hit))
    #local _End = trace(_Box,.5,_Normal+(<rand(Sc2Hair_Rand),rand(Sc2Hair_Rand),rand(Sc2Hair_Rand)>*2-1)/HairArray_Res/2,_Hit);
  #end
  //Scale to fit IHair-Resolution
  #local _Mid=_End*(HairArray_Res-1);
  //Check if we're rounding up or down towards an IHair
  #local _Dec=<(_Mid.x-int(_Mid.x)>.5 ? 0 : 1),(_Mid.y-int(_Mid.y)>.5 ? 0 : 1),(_Mid.z-int(_Mid.z)>.5 ? 0 : 1)>;
  #local _End=_End*(HairArray_Res-1);
  //Ensure that NO value is smaller than 0! (strange Mac)
  #if (_End.x < 0) #local _End = <0,_End.y,_End.z>; #end
  #if (_End.y < 0) #local _End = <_End.x,0,_End.z>; #end
  #if (_End.z < 0) #local _End = <_End.x,_End.y,0>; #end
  //Clip to 3D-Array
  #local _End = <(_Dec.x ? floor(_End.x) : ceil(_End.x)),(_Dec.y ? floor(_End.y) : ceil(_End.y)),(_Dec.z ? floor(_End.z) : ceil(_End.z))>;
  #local _Done=0;
  //3D to 2D
    //pos Y face
    #if (_End.y = HairArray_Res-1)
      #local _Return = <_End.x,_End.z>;
      #local _Done=1;
    #end
    //neg Y face
    #if (_End.y = 0 & !_Done)
      #local _Return = <_End.x,HairArray_Res+_End.z>;
      #local _Done=1;
    #end
    //pos X face
    #if (_End.x = HairArray_Res-1 & !_Done)
      #local _Return = <HairArray_Res-2-_End.y+HairArray_Res,HairArray_Res-1-_End.z>;
      #local _Done=1;
    #end
    //neg X face
    #if (_End.x = 0 & !_Done)
      #local _Return = <_End.y+HairArray_Res*2-3,HairArray_Res-1-_End.z>;
      #local _Done=1;
    #end
    //pos Z face
    #if (_End.z = HairArray_Res-1 & !_Done)
      #local _Return = <_End.x+HairArray_Res-1,HairArray_Res*2-2-_End.y>;
      #local _Done=1;
    #end
    //neg Z face
    #if (_End.z = 0 & !_Done)
      #local _Return = <_End.x+HairArray_Res*2-3,HairArray_Res*2-2-_End.y>;
      #local _Done=1;
    #end
  _Return
#end

//"Index2INormal"
//Function:
// Generates INormal from u/v-node
//Parameters:
// 1. Resolution of ICube
// 2. <u,v> of Index
//Returns:
// 1. INormal
//Notes:
// None
#macro Index2INormal(_IH_Res,_UV)
  #local _X=_UV.u;
  #local _Y=_UV.v;
  #local _Return=<0,0,0>;
  //Generate normals according to section
  //Positive Y face
  #if (_X < _IH_Res & _Y < _IH_Res)
    #local _XCl = _X/(_IH_Res-1);
    #local _YCl = _Y/(_IH_Res-1);
    #local _Return = vnormalize(<_XCl-.5,.5,_YCl-.5>);
  #end
  //Negative Y face
  #if (_X < _IH_Res & _Y-_IH_Res < _IH_Res & _Y >= _IH_Res)
    #local _XCl = _X/(_IH_Res-1);
    #local _YCl = (_Y-_IH_Res)/(_IH_Res-1);
    #local _Return = vnormalize(<_XCl-.5,-.5,_YCl-.5>);
  #end
  //Positive X face
  #if (_X >= _IH_Res & _X < _IH_Res*2-2 & _Y < _IH_Res)
    #local _XCl = (1+(_X-_IH_Res))/(_IH_Res-1);
    #local _YCl = _Y/(_IH_Res-1);
    #local _Return = vnormalize(<.5,.5-_XCl,.5-_YCl>);
  #end
  //Negative X face
  #if (_X >= _IH_Res*2-2 & _Y < _IH_Res)
    #local _XCl = (1+(_X-_IH_Res*2+2))/(_IH_Res-1);
    #local _YCl = _Y/(_IH_Res-1);
    #local _Return = vnormalize(<-.5,_XCl-.5,.5-_YCl>);
  #end
  //Positive Z face
  #if (_X >= _IH_Res & _X < _IH_Res*2-2 & _Y >= _IH_Res & _Y < _IH_Res*2-2)
    #local _XCl = (1+(_X-_IH_Res))/(_IH_Res-1);
    #local _YCl = (1+(_Y-_IH_Res))/(_IH_Res-1);
    #local _Return = vnormalize(<_XCl-.5,.5-_YCl,.5>);
  #end
  //Negative Z face
  #if (_X >= _IH_Res*2-2 & _X < _IH_Res*3-4 & _Y >= _IH_Res & _Y < _IH_Res*2-2)
    #local _XCl = (1+(_X-_IH_Res*2+2))/(_IH_Res-1);
    #local _YCl = (1+(_Y-_IH_Res))/(_IH_Res-1);
    #local _Return = vnormalize(<_XCl-.5,.5-_YCl,-.5>);
  #end
  //Return the INormal
  _Return
#end


//"INormal2Normal"
//Function:
// Calculates transformation to adjust INormal to a Normal for hairs
//Parameters:
// 1. Indexed Normal (unit-length!)
// 2. Desired Normal (unit-length!)
//Returns:
// transform{...}
//Notes:
// None
#macro INormal2Normal(_INormal,_Normal)
  //Calculate Y-Rotation for IHair
  //Normal projected onto y-plane
  #local _nProjY = (abs(_Normal.y)!=1 ? VNormalize(_Normal*<1,0,1>) : -z);
  #local _nInor = (abs(_INormal.y)!=1 ? VNormalize(_INormal*<1,0,1>) : -z);
  #local _yAngle = degrees(acos( vdot(_nProjY,_nInor) ));
  #if (_yAngle!=0)
    //Local Matrix (scaled to avoid precision problems)
    #local _X=vnormalize(vcross(y,_nInor))*100;
    #local _Z=_nInor*100;
    #local _Matrix=transform{matrix <_X.x,_X.y,_X.z,0,100,0,_Z.x,_Z.y,_Z.z,0,0,0>}
    #local _invMatrix=transform{_Matrix inverse}
    #local _F = function{transform{_invMatrix}}
    #local _Q = _F(_Normal.x,_Normal.y,_Normal.z);
    #undef _F
    #local _yAngle = (_Q.x>=0 ? _yAngle : -_yAngle);
  #end //of #if _yAngle!=0
  #if (abs(_Normal.y)=1 & Sc2Hair_YSwitch) #local _yAngle = rand(Sc2Hair_Rand)*360-180; #end
  //Calculate local X-Rotation
  #if (abs(_INormal.y)!=1)
    //Local Matrix
    #local _Z=-_INormal;
    #local _X=vnormalize(vcross(y,_Z));
    #local _Y=vnormalize(vcross(_Z,_X));
    //Normal projected onto local x-plane
    #local _nProjY = vnormalize(_Normal-vdot(_Normal,_X)*_X); 
    #local _nInor = vnormalize(_INormal-vdot(_INormal,_X)*_X); 
    #local _xAngle = degrees(acos( max(min(vdot(_nProjY,_nInor),1),-1) ));
    //Matrix for axis-adjustment needs to be scaled to avoid precision problems
    #local _X=_X*100; #local _Y=_Y*100; #local _Z=_Z*100;
    #local _Matrix=transform{matrix <_X.x,_X.y,_X.z,_Y.x,_Y.y,_Y.z,_Z.x,_Z.y,_Z.z,0,0,0>}
    #local _invMatrix=transform{_Matrix inverse}
    #local _F = function{transform{_invMatrix}}
    #local _Q = _F(_Normal.x,_Normal.y,_Normal.z);
    #undef _F
    #local _xAngle = (_Q.y>=0 ? _xAngle : -_xAngle);
  #else
    //If INormal = +/-y
    #local _Z=vrotate(-z,y*_yAngle);
    #local _X=vrotate(-x,y*_yAngle);
    //Normals projected onto local x-plane
    #local _nProjY = vnormalize(_Normal-vdot(_Normal,_X)*_X); 
    #local _xAngle = degrees(acos( vdot(_nProjY,y) ));
    //Matrix for axis-aligned-rotation
    #local _X=_X*100; #local _Z=_Z*100;
    #local _Matrix=transform{matrix <_X.x,_X.y,_X.z,0,100,0,_Z.x,_Z.y,_Z.z,0,0,0>}
    #local _invMatrix=transform{_Matrix inverse}
    #local _nProjY = _Normal*<1,0,1>;
    #if (_nProjY.z >= 0) #local _xAngle = -_xAngle; #end
  #end
  //The transform!
  transform{
    transform{_invMatrix}
    rotate x*_xAngle
    transform{_Matrix}
    rotate y*_yAngle
  }
#end

//************************************  MACROS TO GENERATE HAIRMESH (And RELATED STUFF)

//"Create_Hairline"
//Function:
// Generates a set of evenly distributed nodes
// to model the path of the hair.
//Parameters:
// 1. Normal
// 2. Length
//Returns:
// 1D-Array with nodes
//Notes:
// Various Hair-Variables are used to specify various parameters
#macro Create_Hairline(_Normal,_Length)
  //Get amount of Steps
  #if (Hair_FixedSteps)
    #local _Steps = Hair_Steps.v;
  #else
    #local _Steps = ceil(_Length*Hair_Steps.v)+2;
  #end
  //Pre-loop variables
  #local _GravMax = Hair_BentPerUnit*1/_Steps*_Length.x;
  #local _StepLength = 1/(_Steps-1)*_Length;
  #local _Array = array[_Steps]
  #local _Array[0] = _Normal*_StepLength;
  #local _A=1;
  #while (_A<_Steps)
    //Interpolator
    #if (vlength(Hair_Wind))
      #local _I = pow(min(_A/_Steps,1),2);
      #local _I = min(Hair_WindStrength,_I);
    #else
      #local _I=0;
    #end
    #if (_I!=1)
      //Gravity
      #local _GravAx = xVnormalize(vcross(-Hair_Gravity,-vnormalize(_Array[_A-1])));
      #local _GravAm = degrees(acos(vdot(VNormalize(Hair_Gravity),vnormalize(_Array[_A-1]))));
      #local _New = vaxis_rotate(_Array[_A-1],_GravAx,-min(_GravMax,_GravAm*vlength(Hair_Gravity)));
      #local _Array[_A]=vnormalize(_New)*_StepLength;
    #else
      #local _Array[_A] = _Array[_A-1];
    #end
   //Wind
   #if (vlength(Hair_Wind))
      #local _Array[_A] = vnormalize(vnormalize(_Array[_A])*(1-_I)+Hair_Wind*_I)*_StepLength;
    #end
    #local _A=_A+1;
  #end
  _Array
#end

//"Create_Hairmesh"
//Function:
// Using the 1D-array's vertices as path, a
// tubular mesh2 is constructed.
//Parameters:
// 1. 1D-Array (as produced by Create_Hairline-Macro)
// 2. <Length, Base_Radius, Tip_Radius> (Length not used)
//Returns:
// Mesh2-Object
//Notes:
// Length is not used, but the vector is simply the one the Sc2Hair-Macros
// will use.
#macro Create_Hairmesh(_HairArray,_Size)
  #local _Detail = Hair_Steps.u;
  #local _Steps = dimension_size(_HairArray,1);
  #local Tube_Nodes = array[_Steps][_Detail][3];
  #local Last_Up = <0,1,0>;
  #local _Pos = <0,0,0>;
  //Creates the vertices/normals/uv for the tube and saves it to
  // an array
  #local C=0;
  #while (C<_Steps)
    //Basic Transformation Data
    #local Actual_Center = _Pos;
    #if (C=0)
      #local New_Direction = vnormalize(_HairArray[C]);
      #local _Axis = <0,1,0>;
      //Value for radius
      #local _Cl=0;
      #local _ClU=0;
      #local _ClR=0;
    #else
      #local _Axis = VNormalize(vcross(vnormalize(_HairArray[C]),vnormalize(_HairArray[C-1])));
      //Only calculate New_Direction when axis has actually changed
      #if (vlength(_Axis))
        #local _Angle = degrees(acos(vdot(vnormalize(_HairArray[C]),vnormalize(_HairArray[C-1]))));
        #local New_Direction = vaxis_rotate(vnormalize(_HairArray[C-1]),_Axis,-_Angle*.5);
      #end
      //Update value for radius 
      #local _Cl = _Cl+vlength(_HairArray[C])/_Size.x;
      #local _ClU = _ClU+vlength(_HairArray[C]);
      #local _ClR = _Cl;
    #end
    //Calculate new local Matrix if needed
    #if (vlength(_Axis))
      #if (C!=0)
        #undef Matrix_Func
        #undef Matrix
      #end
      #local Up = yVnormalize(vcross(vcross(New_Direction,Last_Up), New_Direction));
      #local Last_Up = Up;
      #local Right = xVnormalize(vcross(New_Direction,Up));
      #local _X=-Right;
      #local _Y=Up;
      #local _Z=-New_Direction;
      #if (vdot(Up,_Z)=1 | vdot(Up,_Z)=-1)
        #local _Z=vnormalize(vcross(Right,Up));
      #end
      #local Matrix = 
        transform{
          matrix<_X.x,_X.y,_X.z,
                 _Y.x,_Y.y,_Y.z,
                 _Z.x,_Z.y,_Z.z,
                 0,0,0>
        }
      #local Matrix_Func=function{transform{Matrix}};
    #end
    #local _C=Actual_Center;
    #local Circling = 0;
    #while (Circling < _Detail)
      //Build the Circle
      #local Local_Pos = vrotate(y,z*360*(Circling/_Detail))*<Hair_BaseStretchRatio-(Hair_BaseStretchRatio-Hair_TipStretchRatio)*_Cl,1,1>;
      #local Local_Pos = Matrix_Func(Local_Pos.x,Local_Pos.y,Local_Pos.z);
      //Apply to Array
      #local Tube_Nodes[C][Circling][0] = Local_Pos*(_Size.y-(_Size.y-_Size.z)*_ClR)+_C;
      #if (C!=dimension_size(_HairArray,1)-1 | _Size.z!=0)
        #local Tube_Nodes[C][Circling][1] = vnormalize(Local_Pos);
        #local Tube_Nodes[C][Circling][2] = <(Circling/_Detail),_Cl,0>;
      #else
        #local Tube_Nodes[C][Circling][1] = vnormalize(_HairArray[C]);
        #local Tube_Nodes[C][Circling][2] = <.5,_Cl,0>;
      #end
      #local Circling = Circling + 1;
    #end
    #local _Pos = _Pos + _HairArray[C];
    #local C=C+1;
  #end
  //Now that we have the array, we build the mesh2!
  // Note that when ending on 0, there's no need to "cap" the tip, and thus,
  // less vertices/normals/uv are needed.
  mesh2{
    vertex_vectors{
      #if (_Size.z!=0)
        _Detail*_Steps+2
        #local _A=0;
        #while (_A<_Steps)
          #local _B=0;
          #while (_B<_Detail)
            ,Tube_Nodes[_A][_B][0]
            #local _B=_B+1;
          #end
          #local _A=_A+1;
        #end      
        //Caps
        ,<0,0,0>,_Pos-vnormalize(_HairArray[C-1])*(vlength(_HairArray[C-1])-_Size.z*.5)
      #else
        //Tipped
        _Detail*(_Steps-1)+2
        #local _A=0;
        #while (_A<_Steps-1)
          #local _B=0;
          #while (_B<_Detail)
            ,Tube_Nodes[_A][_B][0]
            #local _B=_B+1;
          #end
          #local _A=_A+1;
        #end      
        //Caps
        ,<0,0,0>,_Pos-_HairArray[C-1]
      #end
    }
    normal_vectors{
      #if (_Size.z!=0)
        _Detail*_Steps+2
        #local _A=0;
        #while (_A<_Steps)
          #local _B=0;
          #while (_B<_Detail)
            ,Tube_Nodes[_A][_B][1]
            #local _B=_B+1;
          #end
          #local _A=_A+1;
        #end      
        //Caps
        ,vnormalize(_HairArray[0]),vnormalize(_HairArray[_Steps-1])
      #else
        //Tipped
        _Detail*(_Steps-1)+2
        #local _A=0;
        #while (_A<_Steps-1)
          #local _B=0;
          #while (_B<_Detail)
            ,Tube_Nodes[_A][_B][1]
            #local _B=_B+1;
          #end
          #local _A=_A+1;
        #end      
        //Caps
        ,-vnormalize(_HairArray[0]),vnormalize(_HairArray[C-1])
      #end
    }
    uv_vectors{
      #if (_Size.z!=0)
        (_Detail+1)*_Steps+2
        #local _A=0;
        #while (_A<_Steps)
          #local _B=0;
          #while (_B<=_Detail)
            #if (_B!=_Detail)
              ,<Tube_Nodes[_A][_B][2].x,Tube_Nodes[_A][_B][2].y>
            #else
              ,<1,Tube_Nodes[_A][_B-1][2].y>
            #end
            #local _B=_B+1;
          #end
          #local _A=_A+1;
        #end      
        //Caps
        ,<.5,0>,<.5,1>
      #else
        //Tipped
        (_Detail+1)*(_Steps-1)+2
        #local _A=0;
        #while (_A<_Steps-1)
          #local _B=0;
          #while (_B<=_Detail)
            #if (_B!=_Detail)
              ,<Tube_Nodes[_A][_B][2].x,Tube_Nodes[_A][_B][2].y>
            #else
              ,<1,Tube_Nodes[_A][_B-1][2].y>
            #end
            #local _B=_B+1;
          #end
          #local _A=_A+1;
        #end      
        //Caps
        ,<.5,0>,<.5,1>
      #end
    }
    face_indices{
      #if (_Size.z!=0)
        _Detail*(_Steps-1)*2+_Detail*2
        #local _A=0;
        #while (_A<_Steps-1)
          #local _B=0;
          #while (_B<_Detail)
            ,<_A*_Detail+_B,(_A+1)*_Detail+_B,_A*_Detail+mod(_B+1,_Detail)>
            ,<_A*_Detail+mod(_B+1,_Detail),(_A+1)*_Detail+_B,(_A+1)*_Detail+mod(_B+1,_Detail)>
            #local _B=_B+1;
          #end
          #local _A=_A+1;
        #end
        //Caps
        #local _B=0;
        #while (_B<_Detail)
          ,<_B,mod(_B+1,_Detail),_Detail*_Steps>
          ,<(_Steps-1)*_Detail+_B,(_Steps-1)*_Detail+mod(_B+1,_Detail),_Detail*_Steps+1>
          #local _B=_B+1;
        #end
      #else
        //Tipped
        _Detail*(_Steps-2)*2+_Detail*2
        #local _A=0;
        #while (_A<_Steps-2)
          #local _B=0;
          #while (_B<_Detail)
            ,<_A*_Detail+_B,(_A+1)*_Detail+_B,_A*_Detail+mod(_B+1,_Detail)>
            ,<_A*_Detail+mod(_B+1,_Detail),(_A+1)*_Detail+_B,(_A+1)*_Detail+mod(_B+1,_Detail)>
            #local _B=_B+1;
          #end
          #local _A=_A+1;
        #end
        //Caps
        #local _B=0;
        #while (_B<_Detail)
          ,<_B,mod(_B+1,_Detail),_Detail*(_Steps-1)>
          ,<(_Steps-2)*_Detail+_B,(_Steps-2)*_Detail+mod(_B+1,_Detail),_Detail*(_Steps-1)+1>
          #local _B=_B+1;
        #end
      #end
    }
    uv_indices{
      #if (_Size.z!=0)
        _Detail*(_Steps-1)*2+_Detail*2
        #local _A=0;
        #while (_A<_Steps-1)
          #local _B=0;
          #while (_B<_Detail)
            ,<_A*(_Detail+1)+_B,(_A+1)*(_Detail+1)+_B,_A*(_Detail+1)+_B+1>
            ,<_A*(_Detail+1)+_B+1,(_A+1)*(_Detail+1)+_B,(_A+1)*(_Detail+1)+_B+1>
            #local _B=_B+1;
          #end
          #local _A=_A+1;
        #end
        //Caps
        #local _B=0;
        #while (_B<_Detail)
          ,<_B,_B+1,(_Detail+1)*_Steps>
          ,<(_Steps-1)*(_Detail+1)+_B,(_Steps-1)*(_Detail+1)+_B+1,(_Detail+1)*_Steps+1>
          #local _B=_B+1;
        #end
      #else
        //Tipped
        _Detail*(_Steps-2)*2+_Detail*2
        #local _A=0;
        #while (_A<_Steps-2)
          #local _B=0;
          #while (_B<_Detail)
            ,<_A*(_Detail+1)+_B,(_A+1)*(_Detail+1)+_B,_A*(_Detail+1)+_B+1>
            ,<_A*(_Detail+1)+_B+1,(_A+1)*(_Detail+1)+_B,(_A+1)*(_Detail+1)+_B+1>
            #local _B=_B+1;
          #end
          #local _A=_A+1;
        #end
        //Caps
        #local _B=0;
        #while (_B<_Detail)
          ,<_B,_B+1,(_Detail+1)*(_Steps-1)>
          ,<(_Steps-2)*(_Detail+1)+_B,(_Steps-2)*(_Detail+1)+_B+1,(_Detail+1)*(_Steps-1)+1>
          #local _B=_B+1;
        #end
      #end
    }
  }
#end

//"Create_INormals"
//Function:
// Generates 2D-Array with clipped normals
//Parameters:
// 1. Resolution for side-length of cube
//Returns:
// 1. 2D-Array with normal-vectors
//Notes:
// None
#macro Create_INormals(_HairArray_Res)
  #local HairArray_Res = _HairArray_Res;
  //Internal
  #local Hair_Normals = array[HairArray_Res*3-4][HairArray_Res*2]
  #local _X=0;
  #while (_X<HairArray_Res*3-4)
    #local _Y=0;
    #while (_Y<HairArray_Res*2)
      #local _Gen=<0,0,0>;
      //Generate normals according to section
      //Positive Y face
      #if (_X < HairArray_Res & _Y < HairArray_Res)
        #local _XCl = _X/(HairArray_Res-1);
        #local _YCl = _Y/(HairArray_Res-1);
        #local Hair_Normals[_X][_Y] = vnormalize(<_XCl-.5,.5,_YCl-.5>);
      #end
      //Negative Y face
      #if (_X < HairArray_Res & _Y-HairArray_Res < HairArray_Res & _Y >= HairArray_Res)
        #local _XCl = _X/(HairArray_Res-1);
        #local _YCl = (_Y-HairArray_Res)/(HairArray_Res-1);
        #local Hair_Normals[_X][_Y] = vnormalize(<_XCl-.5,-.5,_YCl-.5>);
      #end
      //Positive X face
      #if (_X >= HairArray_Res & _X < HairArray_Res*2-2 & _Y < HairArray_Res)
        #local _XCl = (1+(_X-HairArray_Res))/(HairArray_Res-1);
        #local _YCl = _Y/(HairArray_Res-1);
        #local Hair_Normals[_X][_Y] = vnormalize(<.5,.5-_XCl,.5-_YCl>);
      #end
      //Negative X face
      #if (_X >= HairArray_Res*2-2 & _Y < HairArray_Res)
        #local _XCl = (1+(_X-HairArray_Res*2+2))/(HairArray_Res-1);
        #local _YCl = _Y/(HairArray_Res-1);
        #local Hair_Normals[_X][_Y] = vnormalize(<-.5,_XCl-.5,.5-_YCl>);
      #end
      //Positive Z face
      #if (_X >= HairArray_Res & _X < HairArray_Res*2-2 & _Y >= HairArray_Res & _Y < HairArray_Res*2-2)
        #local _XCl = (1+(_X-HairArray_Res))/(HairArray_Res-1);
        #local _YCl = (1+(_Y-HairArray_Res))/(HairArray_Res-1);
        #local Hair_Normals[_X][_Y] = vnormalize(<_XCl-.5,.5-_YCl,.5>);
      #end
      //Negative Z face
      #if (_X >= HairArray_Res*2-2 & _X < HairArray_Res*3-4 & _Y >= HairArray_Res & _Y < HairArray_Res*2-2)
        #local _XCl = (1+(_X-HairArray_Res*2+2))/(HairArray_Res-1);
        #local _YCl = (1+(_Y-HairArray_Res))/(HairArray_Res-1);
        #local Hair_Normals[_X][_Y] = vnormalize(<_XCl-.5,.5-_YCl,-.5>);
      #end
      #local _Y=_Y+1;
    #end
    #local _X=_X+1;
  #end
  //Return the array
  Hair_Normals
#end

//"Create_IHairs"
//Function:
// Given an array with normals, fills a new array with mesh2-hairs
//Parameters:
// 1. Resolution of ICube
// 2. Resolution of IHair-Lengths
// 3. <Base-Radius, Tip-Radius>
// 4. <Min-Length, Max-Length>
//Returns:
// None
//Notes:
// Declares several variables/arrays which will be overwritten the next
// time the macro is called.
#macro Create_IHairs(_IH_Res,_IH_LengthRes,_IH_Radii,_IH_Lengths)
  //Create base-normals
  #local _NorArray = Create_INormals(_IH_Res);
  //Generate length-dimension
  #if (_IH_Lengths.u!=_IH_Lengths.v)
    #local _IHL_Res = _IH_LengthRes;
  #else
    #local _IHL_Res = 1;
  #end
  //Allocate array
  #local _RetArray=array[_IH_Res*3-4][_IH_Res*2][_IHL_Res]
  #if (Surcoat_Debug)
    #debug concat("\nAttempting to declare ",str((pow(_IH_Res,3)-pow(_IH_Res-2,3))*_IHL_Res,0,0)," mesh2-hairs...\n")
  #end
  #local _L=0;
  #while (_L<_IHL_Res)
    //Generate Length-Modifier
    #if (_IHL_Res>1)
      #local _LClock = _L/(_IHL_Res-1);
    #else
      #local _LClock = 0;
    #end
    #local _Size = <_IH_Lengths.u-(_IH_Lengths.u-_IH_Lengths.v)*_LClock,_IH_Radii.u,_IH_Radii.v>;
    //Generate Hairstrand
    #local _X=0;
    #while (_X<_IH_Res*3-4)
      #local _Y=0;
      #while (_Y<_IH_Res*2)
        #ifdef (_NorArray[_X][_Y])
          #local _RetArray[_X][_Y][_L]=Create_IHair(_NorArray[_X][_Y],_Size);
        #end
        #local _Y=_Y+1;
      #end
      #local _X=_X+1;
    #end
    #local _L=_L+1;
  #end
  //Declare the results for Sc2Hair_TestDisplay
  #declare _IH_MeshesArray = _RetArray;
  #declare _IH_MeshesLengths = _IH_Lengths;
  #declare _IH_MeshesLengthRes = _IHL_Res;
  #if (Surcoat_Debug)
    #debug "Done!\n"
  #end
#end

//************************************  MACROS TO DISPLAY & MODIFY END RESULTS

//"SingleHair"
//Function:
// Returns a single hair.
//Parameters:
// 1. Position
// 2. Normal
// 3. Size-Vector (<Length, Base_Radius, Tip_Radius>)
//Returns:
// Mesh2-Object
//Notes:
// None
#macro SingleHair(_Pos,_Nor,_Size)
  object{
    #local _Temp=Create_Hairline(_Nor,_Size.x);
    Create_Hairmesh(_Temp,_Size)
    translate _Pos
  }
#end

//"Sc2Hair_UseCreated"
//Function:
// Visualizes the Surcoat-Samples with hairs generated by
// Create_IHairs
//Parameters:
// 1. Surcoat-Sample-File
//Returns:
// union{} of hairs
//Notes:
// None
#macro Sc2Hair_UseCreated(_CoatInFile)
  #if (Surcoat_Debug)
    #debug concat("\nDisplaying \"",_CoatInFile,"\" with current hairs...\n")
  #end
  //Create some default hairs if User hasn't supplied any
  #ifndef (_IH_MeshesArray)
    #local _WasOn=0;
    #if (Surcoat_Debug)
      #debug " -No hairs found, creating new ones...\n"
      #local _WasOn = 1;
    #end
    #local Surcoat_Debug = off;
    Create_IHairs(4,1,<.1,0>,<1,1>)
    #if (_WasOn) #local Surcoat_Debug = on; #end
    #if (Surcoat_Debug)
      #debug " -Default hairs created\n"
    #end
  #end
  #local _ICube_Res = dimension_size(_IH_MeshesArray,2)/2;
  #local _Load=0;
  #fopen _In _CoatInFile read
  //Loop through all Samples
  union{
    #while (defined(_In))
      //Get data
      #read (_In, _Pos, _Nor)
      #local _Load = _Load + 1;
      #local _O_Nor = _Nor;
      #local _Length = _IH_MeshesLengths.u;
      //Temporary to avoid the User accidentally overwriting the values
      #local _IH_MeshesLengths2=_IH_MeshesLengths;
      //Modify data
      Modify_IHairSample(_Pos,_Nor,_Length,_IH_MeshesLengths2)
      //Generate transformation, pick hair's index from array
      #local Index = Normal2Index(_Nor,_ICube_Res);
      #local INormal=Index2INormal(_ICube_Res,Index);
      #local Trans=INormal2Normal(INormal,_Nor);
      //Pick length from array
      #if (_IH_MeshesLengthRes!=1)
        #local _L_Index = min(floor(max(_Length-_IH_MeshesLengths.u,0)/(_IH_MeshesLengths.v-_IH_MeshesLengths.u)*_IH_MeshesLengthRes),_IH_MeshesLengthRes-1);
        #local _CurrentLength = _IH_MeshesLengths.u-(_IH_MeshesLengths.u-_IH_MeshesLengths.v)*(_L_Index/(_IH_MeshesLengthRes-1));
      #else
        #local _L_Index = 0;
        #local _CurrentLength = _IH_MeshesLengths.u;
      #end
      object{
        _IH_MeshesArray[Index.u][Index.v][_L_Index]
        transform{Trans}
        translate _Pos
        #if (Sc2Hair_IndividualTexture)
          Sc2Hair_Texture(_Pos,_O_Nor,_Nor,_CurrentLength)
        #end
      }
    #end
  }
  #fclose _In
  #if (Surcoat_Debug)
    #debug concat(" -Placed ",str(_Load,0,0)," hairs.\n")
    #debug "Done!\n"
  #end
#end

//"Sc2Hair_UseSaved"
//Function:
// Visualizes the Surcoat-Samples with hairs of given
// Include-File
//Parameters:
// 1. Surcoat-Sample-File
// 2. Write_IHairs-Generated Set of hairs
//Returns:
// union{} of hairs
//Notes:
// If used with "OnTheFly" IHair-Sets, hairs referencing
// a missing IHair will not be placed and a debug-message
// issued.
#macro Sc2Hair_UseSaved(_CoatInFile,_IHairFile)
  #if (Surcoat_Debug)
    #debug concat("\nDisplaying \"",_CoatInFile,"\" with saved hairs...\n")
  #end
  //If Parameter isn't declared, we've never included an IHair-File
  #ifndef (_IH_SavedThisFilename)
    #include _IHairFile
  #end
  //If Parameter is declared and the same as the given Filename, then
  // there's no need to re-load it
  #if (strcmp(_IH_SavedThisFilename,_IHairFile)!=0)
    #include _IHairFile  
  #end
  #local _SparseArray=0;
  #local _ICube_Res = dimension_size(_IH_SavedMeshesArray,2)/2;
  #local _Load=0;
  #fopen _In _CoatInFile read
  //Loop through all Samples
  union{
    #while (defined(_In))
      //Get data
      #read (_In, _Pos, _Nor)
      #local _Load = _Load + 1;
      #local _O_Nor = _Nor;
      #local _Length = _IH_SavedMeshesLengths.u;
      //Temporary to avoid the User accidentally overwriting the values
      #local _IH_SavedMeshesLengths2=_IH_SavedMeshesLengths;
      //Modify data
      Modify_IHairSample(_Pos,_Nor,_Length,_IH_SavedMeshesLengths2)
      //Generate transformation, pick hair's index from array
      #local Index = Normal2Index(_Nor,_ICube_Res);
      #local INormal = Index2INormal(_ICube_Res,Index);
      #local Trans = INormal2Normal(INormal,_Nor);
      //Pick length from array
      #if (_IH_SavedMeshesLengthRes!=1)
        #local _L_Index =
          min(
            floor(
              max(_Length-_IH_SavedMeshesLengths.u,0)
              /(_IH_SavedMeshesLengths.v-_IH_SavedMeshesLengths.u)*_IH_SavedMeshesLengthRes
            ),_IH_SavedMeshesLengthRes-1);
        #local _CurrentLength = _IH_SavedMeshesLengths.u-(_IH_SavedMeshesLengths.u-_IH_SavedMeshesLengths.v)*(_L_Index/(_IH_SavedMeshesLengthRes-1));
      #else
        #local _L_Index = 0;
        #local _CurrentLength = _IH_SavedMeshesLengths.u;
      #end
      #ifndef (_IH_SavedMeshesArray[Index.u][Index.v][_L_Index])
        #local _SparseArray=_SparseArray+1;
      #else
        object{
          _IH_SavedMeshesArray[Index.u][Index.v][_L_Index]
          transform{Trans}
          translate _Pos
          #if (Sc2Hair_IndividualTexture)
            Sc2Hair_Texture(_Pos,_O_Nor,_Nor,_CurrentLength)
          #end
        }
      #end
    #end
  }
  #fclose _In
  #if (Surcoat_Debug)
    #if (_SparseArray) #debug concat(" -The array of hairs wasn't complete, ",str(_SparseArray,0,0)," hairs will be missing.\n") #end
    #debug concat(" -Placed ",str(_Load,0,0)," hairs.\n")
    #debug "Done!\n"
  #end
#end

//"Sc2Hair"
//Function:
// Visualizes the Surcoat-Samples with one hair each
//Parameters:
// 1. Surcoat-Sample-File
// 2. <Base_Radius, Tip_Radius>
// 3. <Min_Length, Max_Length>
//Returns:
// union{} of hairs
//Notes:
// None
// 1. Sc2Hair ( Surcoat_Sample_File, <Base_Rad,Tip_Rad>, <min_Length,max_Length>)
#macro Sc2Hair(_CoatInFile, _Radii, _Lengths)
  #if (Surcoat_Debug)
    #debug concat("\nDisplaying \"",_CoatInFile,"\" with one hair per sample...\n")
  #end
  #local _Load=0;
  #fopen _In _CoatInFile read
  //Loop through all Samples
  union{
    #while (defined(_In))
      //Get data
      #read (_In, _Pos, _Nor)
      #local _Load = _Load + 1;
      #local _O_Nor = _Nor;
      #local _Length = _Lengths.u;
      //Temporary to avoid the User accidentally overwriting the values
      #local _Lengths2=_Lengths;
      //Modify data
      Modify_IHairSample(_Pos,_Nor,_Length,_Lengths2)
      #local _Size = <min(max(_Length,_Lengths.u),_Lengths.v),_Radii.u,_Radii.v>;
      #local _Hair = Create_IHair(_Nor,_Size)
      object{
        _Hair
        translate _Pos
        #if (Sc2Hair_IndividualTexture)
          Sc2Hair_Texture(_Pos,_O_Nor,_Nor,_Length)
        #end
      }
    #end
  }
  #fclose _In
  #if (Surcoat_Debug)
    #debug concat(" -Placed ",str(_Load,0,0)," hairs.\n")
    #debug "Done!\n"
  #end
#end

//************************************  MACROS FOR FILE-I/O

//"Write_Hairmesh"
//Function:
// Same as Create_Hairmesh, but saves to file with
// handle "_sc2hair_FileOut"
//Parameters:
// 1. 1D-Array (as produced by Create_Hairline-Macro)
// 2. <Length, Base_Radius, Tip_Radius> (Length not used)
//Returns:
// Output for Mesh2-Object
//Notes:
// _sc2hair_FileOut has to be declared PRIOR to calling this macro
#macro Write_Hairmesh(_HairArray,_Size)
  //Same as Create_Hairmesh from here...
  #local _Detail = Hair_Steps.u;
  #local _Steps = dimension_size(_HairArray,1);
  #local Tube_Nodes = array[_Steps][_Detail][3];
  #local Last_Up = <0,1,0>;
  #local _Pos = <0,0,0>;
  #local C=0;
  #while (C<_Steps)
    #local Actual_Center = _Pos;
    #if (C=0)
      #local New_Direction = vnormalize(_HairArray[C]);
      #local _Axis = <0,1,0>;
      #local _Cl=0;
      #local _ClU=0;
      #local _ClR=0;
    #else
      #local _Axis = VNormalize(vcross(vnormalize(_HairArray[C]),vnormalize(_HairArray[C-1])));
      #if (vlength(_Axis))
        #local _Angle = degrees(acos(vdot(vnormalize(_HairArray[C]),vnormalize(_HairArray[C-1]))));
        #local New_Direction = vaxis_rotate(vnormalize(_HairArray[C-1]),_Axis,-_Angle*.5);
      #end
      #local _Cl = _Cl+vlength(_HairArray[C])/_Size.x;
      #local _ClU = _ClU+vlength(_HairArray[C]);
      #local _ClR = _Cl;
    #end
    #if (vlength(_Axis))
      #if (C!=0)
        #undef Matrix_Func
        #undef Matrix
      #end
      #local Up = yVnormalize(vcross(vcross(New_Direction,Last_Up), New_Direction));
      #local Last_Up = Up;
      #local Right = xVnormalize(vcross(New_Direction,Up));
      #local _X=-Right;
      #local _Y=Up;
      #local _Z=-New_Direction;
      #if (vdot(Up,_Z)=1 | vdot(Up,_Z)=-1)
        #local _Z=vnormalize(vcross(Right,Up));
      #end
      #local Matrix = 
        transform{
          matrix<_X.x,_X.y,_X.z,
                 _Y.x,_Y.y,_Y.z,
                 _Z.x,_Z.y,_Z.z,
                 0,0,0>
        }
      #local Matrix_Func=function{transform{Matrix}};
    #end
    #local _C=Actual_Center;
    #local Circling = 0;
    #while (Circling < _Detail)
      #local Local_Pos = vrotate(y,z*360*(Circling/_Detail))*<Hair_BaseStretchRatio-(Hair_BaseStretchRatio-Hair_TipStretchRatio)*_Cl,1,1>;
      #local Local_Pos = Matrix_Func(Local_Pos.x,Local_Pos.y,Local_Pos.z);
      #local Tube_Nodes[C][Circling][0] = Local_Pos*(_Size.y-(_Size.y-_Size.z)*_ClR)+_C;
      #if (C!=dimension_size(_HairArray,1)-1 | _Size.z!=0)
        #local Tube_Nodes[C][Circling][1] = vnormalize(Local_Pos);
        #local Tube_Nodes[C][Circling][2] = <(Circling/_Detail),_Cl,0>;
      #else
        #local Tube_Nodes[C][Circling][1] = vnormalize(_HairArray[C]);
        #local Tube_Nodes[C][Circling][2] = <.5,_Cl,0>;
      #end
      #local Circling = Circling + 1;
    #end
    #local _Pos = _Pos + _HairArray[C];
    #local C=C+1;
  #end
  //... to here. Now, instead of parsing a mesh, I write it to _sc2hair_FileOut
  #write (_sc2hair_FileOut, "mesh2{\n")
  #write (_sc2hair_FileOut, "  vertex_vectors{\n")
      #if (_Size.z!=0)
        #write (_sc2hair_FileOut, "    ",str(_Detail*_Steps+2,0,0),"\n")
        #local _A=0;
        #while (_A<_Steps)
          #local _B=0;
          #while (_B<_Detail)
            #write (_sc2hair_FileOut, "    ,",Tube_Nodes[_A][_B][0],"\n")
            #local _B=_B+1;
          #end
          #local _A=_A+1;
        #end      
        //Caps
        #write (_sc2hair_FileOut, "    ,<0,0,0>\n    ,",(_Pos-vnormalize(_HairArray[C-1])*(vlength(_HairArray[C-1])-_Size.z*.5)),"\n")
      #else
        //Tipped
        #write (_sc2hair_FileOut, "    ",str(_Detail*(_Steps-1)+2,0,0),"\n")
        #local _A=0;
        #while (_A<_Steps-1)
          #local _B=0;
          #while (_B<_Detail)
            #write (_sc2hair_FileOut, "    ,",Tube_Nodes[_A][_B][0],"\n")
            #local _B=_B+1;
          #end
          #local _A=_A+1;
        #end      
        //Caps
        #write (_sc2hair_FileOut, "    ,<0,0,0>\n    ,",_Pos-_HairArray[C-1],"\n")
      #end
   #write (_sc2hair_FileOut, "  }\n")
   #write (_sc2hair_FileOut, "  normal_vectors{\n")
      #if (_Size.z!=0)
        #write (_sc2hair_FileOut, "    ",str(_Detail*_Steps+2,0,0),"\n")
        #local _A=0; #while (_A<_Steps)
          #local _B=0; #while (_B<_Detail)
            #write (_sc2hair_FileOut, "    ,",Tube_Nodes[_A][_B][1],"\n")
            #local _B=_B+1; #end
          #local _A=_A+1; #end      
        //Caps
        #write (_sc2hair_FileOut, "    ,",vnormalize(_HairArray[0]),"\n    ,",vnormalize(_HairArray[_Steps-1]),"\n")
      #else
        //Tipped
        #write (_sc2hair_FileOut, "    ",str(_Detail*(_Steps-1)+2,0,0),"\n")
        #local _A=0; #while (_A<_Steps-1)
          #local _B=0; #while (_B<_Detail)
            #write (_sc2hair_FileOut, "    ,",Tube_Nodes[_A][_B][1],"\n")
            #local _B=_B+1; #end
          #local _A=_A+1; #end      
        //Caps
        #write (_sc2hair_FileOut, "    ,",-vnormalize(_HairArray[0]),"\n    ,",vnormalize(_HairArray[C-1]),"\n")
      #end
    #write (_sc2hair_FileOut, "  }\n")
    #write (_sc2hair_FileOut, "  uv_vectors{\n")
      #if (_Size.z!=0)
        #write (_sc2hair_FileOut, "    ",str((_Detail+1)*_Steps+2,0,0),"\n")
        #local _A=0; #while (_A<_Steps)
          #local _B=0; #while (_B<=_Detail)
            #if (_B!=_Detail)
              #write (_sc2hair_FileOut, "    ,",<Tube_Nodes[_A][_B][2].x,Tube_Nodes[_A][_B][2].y>,"\n")
            #else
              #write (_sc2hair_FileOut, "    ,",<1,Tube_Nodes[_A][_B-1][2].y>,"\n")
            #end
            #local _B=_B+1; #end
          #local _A=_A+1; #end      
        //Caps
        #write (_sc2hair_FileOut, "    ,<.5,0>\n    ,<.5,1>\n")
      #else
        //Tipped
        #write (_sc2hair_FileOut, "    ",str((_Detail+1)*(_Steps-1)+2,0,0),"\n")
        #local _A=0; #while (_A<_Steps-1)
          #local _B=0; #while (_B<=_Detail)
            #if (_B!=_Detail)
              #write (_sc2hair_FileOut, "    ,",<Tube_Nodes[_A][_B][2].x,Tube_Nodes[_A][_B][2].y>,"\n")
            #else
              #write (_sc2hair_FileOut, "    ,",<1,Tube_Nodes[_A][_B-1][2].y>,"\n")
            #end
            #local _B=_B+1; #end
          #local _A=_A+1; #end      
        //Caps
        #write (_sc2hair_FileOut, "    ,<.5,0>\n    ,<.5,1>\n")
      #end
    #write (_sc2hair_FileOut, "  }\n")
    #write (_sc2hair_FileOut, "  face_indices{\n")
      #if (_Size.z!=0)
        #write (_sc2hair_FileOut, "    ",str(_Detail*(_Steps-1)*2+_Detail*2,0,0),"\n")
        #local _A=0;
        #while (_A<_Steps-1)
          #local _B=0;
          #while (_B<_Detail)
            #write (_sc2hair_FileOut, "    ,<",str(_A*_Detail+_B,0,0),",",str((_A+1)*_Detail+_B,0,0),",",str(_A*_Detail+mod(_B+1,_Detail),0,0),">\n")
            #write (_sc2hair_FileOut, "    ,<",str(_A*_Detail+mod(_B+1,_Detail),0,0),",",str((_A+1)*_Detail+_B,0,0),",",str((_A+1)*_Detail+mod(_B+1,_Detail),0,0),">\n")
            #local _B=_B+1;
          #end
          #local _A=_A+1;
        #end
        //Caps
        #local _B=0;
        #while (_B<_Detail)
          #write (_sc2hair_FileOut, "    ,<",str(_B,0,0),",",str(mod(_B+1,_Detail),0,0),",",str(_Detail*_Steps,0,0),">\n")
          #write (_sc2hair_FileOut, "    ,<",str((_Steps-1)*_Detail+_B,0,0),",",str((_Steps-1)*_Detail+mod(_B+1,_Detail),0,0),",",str(_Detail*_Steps+1,0,0),">\n")
          #local _B=_B+1;
        #end
      #else
        //Tipped
        #write (_sc2hair_FileOut, "    ",str(_Detail*(_Steps-2)*2+_Detail*2,0,0),"\n")
        #local _A=0;
        #while (_A<_Steps-2)
          #local _B=0;
          #while (_B<_Detail)
            #write (_sc2hair_FileOut, "    ,<",str(_A*_Detail+_B,0,0),",",str((_A+1)*_Detail+_B,0,0),",",str(_A*_Detail+mod(_B+1,_Detail),0,0),">\n")
            #write (_sc2hair_FileOut, "    ,<",str(_A*_Detail+mod(_B+1,_Detail),0,0),",",str((_A+1)*_Detail+_B,0,0),",",str((_A+1)*_Detail+mod(_B+1,_Detail),0,0),">\n")
            #local _B=_B+1;
          #end
          #local _A=_A+1;
        #end
        //Caps
        #local _B=0;
        #while (_B<_Detail)
          #write (_sc2hair_FileOut, "    ,<",str(_B,0,0),",",str(mod(_B+1,_Detail),0,0),",",str(_Detail*(_Steps-1),0,0),">\n")
          #write (_sc2hair_FileOut, "    ,<",str((_Steps-2)*_Detail+_B,0,0),",",str((_Steps-2)*_Detail+mod(_B+1,_Detail),0,0),",",str(_Detail*(_Steps-1)+1,0,0),">\n")
          #local _B=_B+1;
        #end
      #end
    #write (_sc2hair_FileOut, "  }\n")
    #write (_sc2hair_FileOut, "  uv_indices{\n")
      #if (_Size.z!=0)
        #write (_sc2hair_FileOut, "    ",str(_Detail*(_Steps-1)*2+_Detail*2,0,0),"\n")
        #local _A=0;
        #while (_A<_Steps-1)
          #local _B=0;
          #while (_B<_Detail)
            #write (_sc2hair_FileOut, "    ,<",str(_A*(_Detail+1)+_B,0,0),",",str((_A+1)*(_Detail+1)+_B,0,0),",",str(_A*(_Detail+1)+_B+1,0,0),">\n")
            #write (_sc2hair_FileOut, "    ,<",str(_A*(_Detail+1)+_B+1,0,0),",",str((_A+1)*(_Detail+1)+_B,0,0),",",str((_A+1)*(_Detail+1)+_B+1,0,0),">\n")
            #local _B=_B+1;
          #end
          #local _A=_A+1;
        #end
        //Caps
        #local _B=0;
        #while (_B<_Detail)
          #write (_sc2hair_FileOut, "    ,<",str(_B,0,0),",",str(_B+1,0,0),",",str((_Detail+1)*_Steps,0,0),">\n")
          #write (_sc2hair_FileOut, "    ,<",str((_Steps-1)*(_Detail+1)+_B,0,0),",",str((_Steps-1)*(_Detail+1)+_B+1,0,0),",",str((_Detail+1)*_Steps+1,0,0),">\n")
          #local _B=_B+1;
        #end
      #else
        //Tipped
        #write (_sc2hair_FileOut, "    ",str(_Detail*(_Steps-2)*2+_Detail*2,0,0),"\n")
        #local _A=0;
        #while (_A<_Steps-2)
          #local _B=0;
          #while (_B<_Detail)
            #write (_sc2hair_FileOut, "    ,<",str(_A*(_Detail+1)+_B,0,0),",",str((_A+1)*(_Detail+1)+_B,0,0),",",str(_A*(_Detail+1)+_B+1,0,0),">\n")
            #write (_sc2hair_FileOut, "    ,<",str(_A*(_Detail+1)+_B+1,0,0),",",str((_A+1)*(_Detail+1)+_B,0,0),",",str((_A+1)*(_Detail+1)+_B+1,0,0),">\n")
            #local _B=_B+1;
          #end
          #local _A=_A+1;
        #end
        //Caps
        #local _B=0;
        #while (_B<_Detail)
          #write (_sc2hair_FileOut, "    ,<",str(_B,0,0),",",str(_B+1,0,0),",",str((_Detail+1)*(_Steps-1),0,0),">\n")
          #write (_sc2hair_FileOut, "    ,<",str((_Steps-2)*(_Detail+1)+_B,0,0),",",str((_Steps-2)*(_Detail+1)+_B+1,0,0),",",str((_Detail+1)*(_Steps-1)+1,0,0),">\n")
          #local _B=_B+1;
        #end
      #end
  #write (_sc2hair_FileOut, "  }\n")
  #write (_sc2hair_FileOut, "}\n\n")
#end

//"Write_IHairs"
//Function:
// Given an array with normals, writes to disk a new array with mesh2-hairs
//Parameters:
// 1. Resolution of ICube
// 2. Resolution of IHair-Lengths
// 3. <Base-Radius, Tip-Radius>
// 4. <Min-Length, Max-Length>
// 5. Filename
//Returns:
// None, but saves array with mesh2-hairs to file
//Notes:
// None
#macro Write_IHairs(_IH_Res,_IH_LengthRes,_IH_Radii,_IH_Lengths,_OutFilename)
  #if (Surcoat_Debug)
    #debug concat("\nWriting hair to \"",_OutFilename,"\", this may take some time...\n")
  #end
  //Open File for output
  #fopen _sc2hair_FileOut _OutFilename write
  #write (_sc2hair_FileOut, "//IHair-Set, created with Tim Nikias' Surcoat2Hair Macros\n")
  #write (_sc2hair_FileOut, "#declare _IH_SavedThisFilename=\"",_OutFilename,"\"\n")
  //Generate length-dimension
  #if (_IH_Lengths.u!=_IH_Lengths.v)
    #local _IHL_Res = _IH_LengthRes;
  #else
    #local _IHL_Res = 1;
  #end
  //Saves Stats
  #write (_sc2hair_FileOut, "#declare _IH_SavedMeshesArray=array[",str(_IH_Res*3-4,0,0),"][",str(_IH_Res*2,0,0),"][",str(_IHL_Res,0,0),"]\n")
  #write (_sc2hair_FileOut, "#declare _IH_SavedMeshesLengths=<",_IH_Lengths.u,",",_IH_Lengths.v,">;\n")
  #write (_sc2hair_FileOut, "#declare _IH_SavedMeshesLengthRes=",_IH_LengthRes,";\n")
  //Create base-normals
  #local _NorArray = Create_INormals(_IH_Res);
  #local _L=0;
  #while (_L<_IHL_Res)
    //Generate Length-Modifier
    #if (_IHL_Res>1)
      #local _LClock = _L/(_IHL_Res-1);
    #else
      #local _LClock = 0;
    #end
    #local _Size = <_IH_Lengths.u-(_IH_Lengths.u-_IH_Lengths.v)*_LClock,_IH_Radii.u,_IH_Radii.v>;
    //Generate Hairstrand
    #local _X=0;
    #while (_X<_IH_Res*3-4)
      #local _Y=0;
      #while (_Y<_IH_Res*2)
        #ifdef (_NorArray[_X][_Y])
          #write (_sc2hair_FileOut, "#declare _IH_SavedMeshesArray[",str(_X,0,0),"][",str(_Y,0,0),"][",str(_L,0,0),"]=\n")
          Write_IHair(_NorArray[_X][_Y],_Size)
          #write (_sc2hair_FileOut, "\n")
        #end
        #local _Y=_Y+1;
      #end
      #local _X=_X+1;
    #end
    #local _L=_L+1;
  #end
  //Close the file
  #write (_sc2hair_FileOut, "//End of File\n")
  #fclose _sc2hair_FileOut
  #if (Surcoat_Debug)
    #debug "Done!\n"
  #end
#end

//"OnTheFly_Hairmesh"
//Function:
// Mix of Create- & Write_Hairmesh, saves to file with
// handle "_sc2hair_FileOut" as well as returning mesh2-object.
//Parameters:
// 1. 1D-Array (as produced by Create_Hairline-Macro)
// 2. <Length, Base_Radius, Tip_Radius> (Length not used)
//Returns:
// Output for Mesh2-Object and mesh2-object
//Notes:
// _sc2hair_FileOut has to be declared PRIOR to calling this macro
#macro OnTheFly_Hairmesh(_HairArray,_Size)
  //Same as Create_Hairmesh from here...
  #local _Detail = Hair_Steps.u;
  #local _Steps = dimension_size(_HairArray,1);
  #local Tube_Nodes = array[_Steps][_Detail][3];
  #local Last_Up = <0,1,0>;
  #local _Pos = <0,0,0>;
  #local C=0;
  #while (C<_Steps)
    #local Actual_Center = _Pos;
    #if (C=0)
      #local New_Direction = vnormalize(_HairArray[C]);
      #local _Axis = <0,1,0>;
      #local _Cl=0;
      #local _ClU=0;
      #local _ClR=0;
    #else
      #local _Axis = VNormalize(vcross(vnormalize(_HairArray[C]),vnormalize(_HairArray[C-1])));
      #if (vlength(_Axis))
        #local _Angle = degrees(acos(vdot(vnormalize(_HairArray[C]),vnormalize(_HairArray[C-1]))));
        #local New_Direction = vaxis_rotate(vnormalize(_HairArray[C-1]),_Axis,-_Angle*.5);
      #end
      #local _Cl = _Cl+vlength(_HairArray[C])/_Size.x;
      #local _ClU = _ClU+vlength(_HairArray[C]);
      #local _ClR = _Cl;
    #end
    #if (vlength(_Axis))
      #if (C!=0)
        #undef Matrix_Func
        #undef Matrix
      #end
      #local Up = yVnormalize(vcross(vcross(New_Direction,Last_Up), New_Direction));
      #local Last_Up = Up;
      #local Right = xVnormalize(vcross(New_Direction,Up));
      #local _X=-Right;
      #local _Y=Up;
      #local _Z=-New_Direction;
      #if (vdot(Up,_Z)=1 | vdot(Up,_Z)=-1)
        #local _Z=vnormalize(vcross(Right,Up));
      #end
      #local Matrix = 
        transform{
          matrix<_X.x,_X.y,_X.z,
                 _Y.x,_Y.y,_Y.z,
                 _Z.x,_Z.y,_Z.z,
                 0,0,0>
        }
      #local Matrix_Func=function{transform{Matrix}};
    #end
    #local _C=Actual_Center;
    #local Circling = 0;
    #while (Circling < _Detail)
      #local Local_Pos = vrotate(y,z*360*(Circling/_Detail))*<Hair_BaseStretchRatio-(Hair_BaseStretchRatio-Hair_TipStretchRatio)*_Cl,1,1>;
      #local Local_Pos = Matrix_Func(Local_Pos.x,Local_Pos.y,Local_Pos.z);
      #local Tube_Nodes[C][Circling][0] = Local_Pos*(_Size.y-(_Size.y-_Size.z)*_ClR)+_C;
      #if (C!=dimension_size(_HairArray,1)-1 | _Size.z!=0)
        #local Tube_Nodes[C][Circling][1] = vnormalize(Local_Pos);
        #local Tube_Nodes[C][Circling][2] = <(Circling/_Detail),_Cl,0>;
      #else
        #local Tube_Nodes[C][Circling][1] = vnormalize(_HairArray[C]);
        #local Tube_Nodes[C][Circling][2] = <.5,_Cl,0>;
      #end
      #local Circling = Circling + 1;
    #end
    #local _Pos = _Pos + _HairArray[C];
    #local C=C+1;
  #end
  //... to here. Now, instead of parsing a mesh, I write it to _sc2hair_FileOut
  #write (_sc2hair_FileOut, "mesh2{\n")
  #write (_sc2hair_FileOut, "  vertex_vectors{\n")
      #if (_Size.z!=0)
        #write (_sc2hair_FileOut, "    ",str(_Detail*_Steps+2,0,0),"\n")
        #local _A=0;
        #while (_A<_Steps)
          #local _B=0;
          #while (_B<_Detail)
            #write (_sc2hair_FileOut, "    ,",Tube_Nodes[_A][_B][0],"\n")
            #local _B=_B+1;
          #end
          #local _A=_A+1;
        #end      
        //Caps
        #write (_sc2hair_FileOut, "    ,<0,0,0>\n    ,",(_Pos-vnormalize(_HairArray[C-1])*(vlength(_HairArray[C-1])-_Size.z*.5)),"\n")
      #else
        //Tipped
        #write (_sc2hair_FileOut, "    ",str(_Detail*(_Steps-1)+2,0,0),"\n")
        #local _A=0;
        #while (_A<_Steps-1)
          #local _B=0;
          #while (_B<_Detail)
            #write (_sc2hair_FileOut, "    ,",Tube_Nodes[_A][_B][0],"\n")
            #local _B=_B+1;
          #end
          #local _A=_A+1;
        #end      
        //Caps
        #write (_sc2hair_FileOut, "    ,<0,0,0>\n    ,",_Pos-_HairArray[C-1],"\n")
      #end
   #write (_sc2hair_FileOut, "  }\n")
   #write (_sc2hair_FileOut, "  normal_vectors{\n")
      #if (_Size.z!=0)
        #write (_sc2hair_FileOut, "    ",str(_Detail*_Steps+2,0,0),"\n")
        #local _A=0; #while (_A<_Steps)
          #local _B=0; #while (_B<_Detail)
            #write (_sc2hair_FileOut, "    ,",Tube_Nodes[_A][_B][1],"\n")
            #local _B=_B+1; #end
          #local _A=_A+1; #end      
        //Caps
        #write (_sc2hair_FileOut, "    ,",vnormalize(_HairArray[0]),"\n    ,",vnormalize(_HairArray[_Steps-1]),"\n")
      #else
        //Tipped
        #write (_sc2hair_FileOut, "    ",str(_Detail*(_Steps-1)+2,0,0),"\n")
        #local _A=0; #while (_A<_Steps-1)
          #local _B=0; #while (_B<_Detail)
            #write (_sc2hair_FileOut, "    ,",Tube_Nodes[_A][_B][1],"\n")
            #local _B=_B+1; #end
          #local _A=_A+1; #end      
        //Caps
        #write (_sc2hair_FileOut, "    ,",-vnormalize(_HairArray[0]),"\n    ,",vnormalize(_HairArray[C-1]),"\n")
      #end
    #write (_sc2hair_FileOut, "  }\n")
    #write (_sc2hair_FileOut, "  uv_vectors{\n")
      #if (_Size.z!=0)
        #write (_sc2hair_FileOut, "    ",str((_Detail+1)*_Steps+2,0,0),"\n")
        #local _A=0; #while (_A<_Steps)
          #local _B=0; #while (_B<=_Detail)
            #if (_B!=_Detail)
              #write (_sc2hair_FileOut, "    ,",<Tube_Nodes[_A][_B][2].x,Tube_Nodes[_A][_B][2].y>,"\n")
            #else
              #write (_sc2hair_FileOut, "    ,",<1,Tube_Nodes[_A][_B-1][2].y>,"\n")
            #end
            #local _B=_B+1; #end
          #local _A=_A+1; #end      
        //Caps
        #write (_sc2hair_FileOut, "    ,<.5,0>\n    ,<.5,1>\n")
      #else
        //Tipped
        #write (_sc2hair_FileOut, "    ",str((_Detail+1)*(_Steps-1)+2,0,0),"\n")
        #local _A=0; #while (_A<_Steps-1)
          #local _B=0; #while (_B<=_Detail)
            #if (_B!=_Detail)
              #write (_sc2hair_FileOut, "    ,",<Tube_Nodes[_A][_B][2].x,Tube_Nodes[_A][_B][2].y>,"\n")
            #else
              #write (_sc2hair_FileOut, "    ,",<1,Tube_Nodes[_A][_B-1][2].y>,"\n")
            #end
            #local _B=_B+1; #end
          #local _A=_A+1; #end      
        //Caps
        #write (_sc2hair_FileOut, "    ,<.5,0>\n    ,<.5,1>\n")
      #end
    #write (_sc2hair_FileOut, "  }\n")
    #write (_sc2hair_FileOut, "  face_indices{\n")
      #if (_Size.z!=0)
        #write (_sc2hair_FileOut, "    ",str(_Detail*(_Steps-1)*2+_Detail*2,0,0),"\n")
        #local _A=0;
        #while (_A<_Steps-1)
          #local _B=0;
          #while (_B<_Detail)
            #write (_sc2hair_FileOut, "    ,<",str(_A*_Detail+_B,0,0),",",str((_A+1)*_Detail+_B,0,0),",",str(_A*_Detail+mod(_B+1,_Detail),0,0),">\n")
            #write (_sc2hair_FileOut, "    ,<",str(_A*_Detail+mod(_B+1,_Detail),0,0),",",str((_A+1)*_Detail+_B,0,0),",",str((_A+1)*_Detail+mod(_B+1,_Detail),0,0),">\n")
            #local _B=_B+1;
          #end
          #local _A=_A+1;
        #end
        //Caps
        #local _B=0;
        #while (_B<_Detail)
          #write (_sc2hair_FileOut, "    ,<",str(_B,0,0),",",str(mod(_B+1,_Detail),0,0),",",str(_Detail*_Steps,0,0),">\n")
          #write (_sc2hair_FileOut, "    ,<",str((_Steps-1)*_Detail+_B,0,0),",",str((_Steps-1)*_Detail+mod(_B+1,_Detail),0,0),",",str(_Detail*_Steps+1,0,0),">\n")
          #local _B=_B+1;
        #end
      #else
        //Tipped
        #write (_sc2hair_FileOut, "    ",str(_Detail*(_Steps-2)*2+_Detail*2,0,0),"\n")
        #local _A=0;
        #while (_A<_Steps-2)
          #local _B=0;
          #while (_B<_Detail)
            #write (_sc2hair_FileOut, "    ,<",str(_A*_Detail+_B,0,0),",",str((_A+1)*_Detail+_B,0,0),",",str(_A*_Detail+mod(_B+1,_Detail),0,0),">\n")
            #write (_sc2hair_FileOut, "    ,<",str(_A*_Detail+mod(_B+1,_Detail),0,0),",",str((_A+1)*_Detail+_B,0,0),",",str((_A+1)*_Detail+mod(_B+1,_Detail),0,0),">\n")
            #local _B=_B+1;
          #end
          #local _A=_A+1;
        #end
        //Caps
        #local _B=0;
        #while (_B<_Detail)
          #write (_sc2hair_FileOut, "    ,<",str(_B,0,0),",",str(mod(_B+1,_Detail),0,0),",",str(_Detail*(_Steps-1),0,0),">\n")
          #write (_sc2hair_FileOut, "    ,<",str((_Steps-2)*_Detail+_B,0,0),",",str((_Steps-2)*_Detail+mod(_B+1,_Detail),0,0),",",str(_Detail*(_Steps-1)+1,0,0),">\n")
          #local _B=_B+1;
        #end
      #end
    #write (_sc2hair_FileOut, "  }\n")
    #write (_sc2hair_FileOut, "  uv_indices{\n")
      #if (_Size.z!=0)
        #write (_sc2hair_FileOut, "    ",str(_Detail*(_Steps-1)*2+_Detail*2,0,0),"\n")
        #local _A=0;
        #while (_A<_Steps-1)
          #local _B=0;
          #while (_B<_Detail)
            #write (_sc2hair_FileOut, "    ,<",str(_A*(_Detail+1)+_B,0,0),",",str((_A+1)*(_Detail+1)+_B,0,0),",",str(_A*(_Detail+1)+_B+1,0,0),">\n")
            #write (_sc2hair_FileOut, "    ,<",str(_A*(_Detail+1)+_B+1,0,0),",",str((_A+1)*(_Detail+1)+_B,0,0),",",str((_A+1)*(_Detail+1)+_B+1,0,0),">\n")
            #local _B=_B+1;
          #end
          #local _A=_A+1;
        #end
        //Caps
        #local _B=0;
        #while (_B<_Detail)
          #write (_sc2hair_FileOut, "    ,<",str(_B,0,0),",",str(_B+1,0,0),",",str((_Detail+1)*_Steps,0,0),">\n")
          #write (_sc2hair_FileOut, "    ,<",str((_Steps-1)*(_Detail+1)+_B,0,0),",",str((_Steps-1)*(_Detail+1)+_B+1,0,0),",",str((_Detail+1)*_Steps+1,0,0),">\n")
          #local _B=_B+1;
        #end
      #else
        //Tipped
        #write (_sc2hair_FileOut, "    ",str(_Detail*(_Steps-2)*2+_Detail*2,0,0),"\n")
        #local _A=0;
        #while (_A<_Steps-2)
          #local _B=0;
          #while (_B<_Detail)
            #write (_sc2hair_FileOut, "    ,<",str(_A*(_Detail+1)+_B,0,0),",",str((_A+1)*(_Detail+1)+_B,0,0),",",str(_A*(_Detail+1)+_B+1,0,0),">\n")
            #write (_sc2hair_FileOut, "    ,<",str(_A*(_Detail+1)+_B+1,0,0),",",str((_A+1)*(_Detail+1)+_B,0,0),",",str((_A+1)*(_Detail+1)+_B+1,0,0),">\n")
            #local _B=_B+1;
          #end
          #local _A=_A+1;
        #end
        //Caps
        #local _B=0;
        #while (_B<_Detail)
          #write (_sc2hair_FileOut, "    ,<",str(_B,0,0),",",str(_B+1,0,0),",",str((_Detail+1)*(_Steps-1),0,0),">\n")
          #write (_sc2hair_FileOut, "    ,<",str((_Steps-2)*(_Detail+1)+_B,0,0),",",str((_Steps-2)*(_Detail+1)+_B+1,0,0),",",str((_Detail+1)*(_Steps-1)+1,0,0),">\n")
          #local _B=_B+1;
        #end
      #end
  #write (_sc2hair_FileOut, "  }\n")
  #write (_sc2hair_FileOut, "}\n\n")
  //Now that we have the array, we build the mesh2!
  // Note that when ending on 0, there's no need to "cap" the tip, and thus,
  // less vertices/normals/uv are needed.
  mesh2{
    vertex_vectors{
      #if (_Size.z!=0)
        _Detail*_Steps+2
        #local _A=0;
        #while (_A<_Steps)
          #local _B=0;
          #while (_B<_Detail)
            ,Tube_Nodes[_A][_B][0]
            #local _B=_B+1;
          #end
          #local _A=_A+1;
        #end      
        //Caps
        ,<0,0,0>,_Pos-vnormalize(_HairArray[C-1])*(vlength(_HairArray[C-1])-_Size.z*.5)
      #else
        //Tipped
        _Detail*(_Steps-1)+2
        #local _A=0;
        #while (_A<_Steps-1)
          #local _B=0;
          #while (_B<_Detail)
            ,Tube_Nodes[_A][_B][0]
            #local _B=_B+1;
          #end
          #local _A=_A+1;
        #end      
        //Caps
        ,<0,0,0>,_Pos-_HairArray[C-1]
      #end
    }
    normal_vectors{
      #if (_Size.z!=0)
        _Detail*_Steps+2
        #local _A=0;
        #while (_A<_Steps)
          #local _B=0;
          #while (_B<_Detail)
            ,Tube_Nodes[_A][_B][1]
            #local _B=_B+1;
          #end
          #local _A=_A+1;
        #end      
        //Caps
        ,vnormalize(_HairArray[0]),vnormalize(_HairArray[_Steps-1])
      #else
        //Tipped
        _Detail*(_Steps-1)+2
        #local _A=0;
        #while (_A<_Steps-1)
          #local _B=0;
          #while (_B<_Detail)
            ,Tube_Nodes[_A][_B][1]
            #local _B=_B+1;
          #end
          #local _A=_A+1;
        #end      
        //Caps
        ,-vnormalize(_HairArray[0]),vnormalize(_HairArray[C-1])
      #end
    }
    uv_vectors{
      #if (_Size.z!=0)
        (_Detail+1)*_Steps+2
        #local _A=0;
        #while (_A<_Steps)
          #local _B=0;
          #while (_B<=_Detail)
            #if (_B!=_Detail)
              ,<Tube_Nodes[_A][_B][2].x,Tube_Nodes[_A][_B][2].y>
            #else
              ,<1,Tube_Nodes[_A][_B-1][2].y>
            #end
            #local _B=_B+1;
          #end
          #local _A=_A+1;
        #end      
        //Caps
        ,<.5,0>,<.5,1>
      #else
        //Tipped
        (_Detail+1)*(_Steps-1)+2
        #local _A=0;
        #while (_A<_Steps-1)
          #local _B=0;
          #while (_B<=_Detail)
            #if (_B!=_Detail)
              ,<Tube_Nodes[_A][_B][2].x,Tube_Nodes[_A][_B][2].y>
            #else
              ,<1,Tube_Nodes[_A][_B-1][2].y>
            #end
            #local _B=_B+1;
          #end
          #local _A=_A+1;
        #end      
        //Caps
        ,<.5,0>,<.5,1>
      #end
    }
    face_indices{
      #if (_Size.z!=0)
        _Detail*(_Steps-1)*2+_Detail*2
        #local _A=0;
        #while (_A<_Steps-1)
          #local _B=0;
          #while (_B<_Detail)
            ,<_A*_Detail+_B,(_A+1)*_Detail+_B,_A*_Detail+mod(_B+1,_Detail)>
            ,<_A*_Detail+mod(_B+1,_Detail),(_A+1)*_Detail+_B,(_A+1)*_Detail+mod(_B+1,_Detail)>
            #local _B=_B+1;
          #end
          #local _A=_A+1;
        #end
        //Caps
        #local _B=0;
        #while (_B<_Detail)
          ,<_B,mod(_B+1,_Detail),_Detail*_Steps>
          ,<(_Steps-1)*_Detail+_B,(_Steps-1)*_Detail+mod(_B+1,_Detail),_Detail*_Steps+1>
          #local _B=_B+1;
        #end
      #else
        //Tipped
        _Detail*(_Steps-2)*2+_Detail*2
        #local _A=0;
        #while (_A<_Steps-2)
          #local _B=0;
          #while (_B<_Detail)
            ,<_A*_Detail+_B,(_A+1)*_Detail+_B,_A*_Detail+mod(_B+1,_Detail)>
            ,<_A*_Detail+mod(_B+1,_Detail),(_A+1)*_Detail+_B,(_A+1)*_Detail+mod(_B+1,_Detail)>
            #local _B=_B+1;
          #end
          #local _A=_A+1;
        #end
        //Caps
        #local _B=0;
        #while (_B<_Detail)
          ,<_B,mod(_B+1,_Detail),_Detail*(_Steps-1)>
          ,<(_Steps-2)*_Detail+_B,(_Steps-2)*_Detail+mod(_B+1,_Detail),_Detail*(_Steps-1)+1>
          #local _B=_B+1;
        #end
      #end
    }
    uv_indices{
      #if (_Size.z!=0)
        _Detail*(_Steps-1)*2+_Detail*2
        #local _A=0;
        #while (_A<_Steps-1)
          #local _B=0;
          #while (_B<_Detail)
            ,<_A*(_Detail+1)+_B,(_A+1)*(_Detail+1)+_B,_A*(_Detail+1)+_B+1>
            ,<_A*(_Detail+1)+_B+1,(_A+1)*(_Detail+1)+_B,(_A+1)*(_Detail+1)+_B+1>
            #local _B=_B+1;
          #end
          #local _A=_A+1;
        #end
        //Caps
        #local _B=0;
        #while (_B<_Detail)
          ,<_B,_B+1,(_Detail+1)*_Steps>
          ,<(_Steps-1)*(_Detail+1)+_B,(_Steps-1)*(_Detail+1)+_B+1,(_Detail+1)*_Steps+1>
          #local _B=_B+1;
        #end
      #else
        //Tipped
        _Detail*(_Steps-2)*2+_Detail*2
        #local _A=0;
        #while (_A<_Steps-2)
          #local _B=0;
          #while (_B<_Detail)
            ,<_A*(_Detail+1)+_B,(_A+1)*(_Detail+1)+_B,_A*(_Detail+1)+_B+1>
            ,<_A*(_Detail+1)+_B+1,(_A+1)*(_Detail+1)+_B,(_A+1)*(_Detail+1)+_B+1>
            #local _B=_B+1;
          #end
          #local _A=_A+1;
        #end
        //Caps
        #local _B=0;
        #while (_B<_Detail)
          ,<_B,_B+1,(_Detail+1)*(_Steps-1)>
          ,<(_Steps-2)*(_Detail+1)+_B,(_Steps-2)*(_Detail+1)+_B+1,(_Detail+1)*(_Steps-1)+1>
          #local _B=_B+1;
        #end
      #end
    }
  }
#end

//"Sc2Hair_OnTheFly"
//Function:
// Given an array with normals, fills a new array with mesh2-hairs. Only
// creates those hairs needed by the given sample file.
//Parameters:
// 1. Resolution of ICube
// 2. Resolution of IHair-Lengths
// 3. <Base-Radius, Tip-Radius>
// 4. <Min-Length, Max-Length>
// 5. Filename
//Returns:
// Array with mesh2-hairs
//Notes:
// None
#macro Sc2Hair_OnTheFly(_CoatInFile,_IH_Res,_IH_LengthRes,_IH_Radii,_IH_Lengths,_OutFilename)
  #if (Surcoat_Debug)
    #debug concat("\nCreating hair on-the-fly and writing to \"",_OutFilename,"\", this may take some time...\n")
  #end
  //Open File for output
  #fopen _sc2hair_FileOut _OutFilename write
  #write (_sc2hair_FileOut, "//IHair-Set, created with Tim Nikias' Surcoat2Hair Macros\n")
  #write (_sc2hair_FileOut, "#declare _IH_SavedThisFilename=\"",_OutFilename,"\"\n")
  //Generate length-dimension
  #if (_IH_Lengths.u!=_IH_Lengths.v)
    #local _IHL_Res = _IH_LengthRes;
  #else
    #local _IHL_Res = 1;
  #end
  //Saves Stats
  #local _IH_Tracker = array[_IH_Res*3-4][_IH_Res*2][_IHL_Res]
  #local _WrittenIHairs = 0;
  #declare _IH_SavedMeshesArray = array[_IH_Res*3-4][_IH_Res*2][_IHL_Res]
  #declare _IH_SavedMeshesLengths=_IH_Lengths;
  #declare _IH_SavedMeshesLengthRes=_IH_LengthRes;
  #write (_sc2hair_FileOut, "#declare _IH_SavedMeshesArray=array[",str(_IH_Res*3-4,0,0),"][",str(_IH_Res*2,0,0),"][",str(_IHL_Res,0,0),"]\n")
  #write (_sc2hair_FileOut, "#declare _IH_SavedMeshesLengths=<",_IH_Lengths.u,",",_IH_Lengths.v,">;\n")
  #write (_sc2hair_FileOut, "#declare _IH_SavedMeshesLengthRes=",_IH_LengthRes,";\n")
  //Create base-normals
  #local _NorArray = Create_INormals(_IH_Res);
  #local _ICube_Res = _IH_Res;
  #local _Load=0;
  #fopen _In _CoatInFile read
  //Loop through all Samples
  union{
    #while (defined(_In))
      //Get data
      #read (_In, _Pos, _Nor)
      #local _Load = _Load + 1;
      #local _O_Nor = _Nor;
      #local _Length = _IH_Lengths.u;
      //Temporary to avoid the User accidentally overwriting the values
      #local _IH_MeshesLengths2=_IH_Lengths;
      //Modify data
      Modify_IHairSample(_Pos,_Nor,_Length,_IH_MeshesLengths2)
      //Generate transformation, pick hair's index from array
      #local Index = Normal2Index(_Nor,_ICube_Res);
      #local INormal = Index2INormal(_ICube_Res,Index);
      #local Trans = INormal2Normal(INormal,_Nor);
      //Pick length from array
      #if (_IH_LengthRes!=1)
        #local _L_Index = min(floor(max(_Length-_IH_Lengths.u,0)/(_IH_Lengths.v-_IH_Lengths.u)*_IH_LengthRes),_IH_LengthRes-1);
        #local _CurrentLength = _IH_Lengths.u-(_IH_Lengths.u-_IH_Lengths.v)*(_L_Index/(_IH_LengthRes-1));
      #else
        #local _L_Index = 0;
        #local _CurrentLength = _IH_Lengths.u;
      #end
      //Generate hair as well as save it if it's not been created yet!
      #ifndef(_IH_Tracker[Index.u][Index.v][_L_Index])
        #local _Size = <_CurrentLength,_IH_Radii.u,_IH_Radii.v>;
        #local _IH_Tracker[Index.u][Index.v][_L_Index] = 1;
        #local _WrittenIHairs = _WrittenIHairs+1;
        #write (_sc2hair_FileOut, "#declare _IH_SavedMeshesArray[",str(Index.u,0,0),"][",str(Index.v,0,0),"][",str(_L_Index,0,0),"]=\n")
        #declare _IH_SavedMeshesArray[Index.u][Index.v][_L_Index]=OnTheFly_IHair(_NorArray[Index.u][Index.v],_Size);
        #write (_sc2hair_FileOut, "\n")
      #end
      object{
        _IH_SavedMeshesArray[Index.u][Index.v][_L_Index]
        transform{Trans}
        translate _Pos
        #if (Sc2Hair_IndividualTexture)
          Sc2Hair_Texture(_Pos,_O_Nor,_Nor,_CurrentLength)
        #end
      }
      #undef Trans
    #end
  }
  #fclose _In
  //Close the file
  #write (_sc2hair_FileOut, "//End of File\n")
  #fclose _sc2hair_FileOut
  #if (Surcoat_Debug)
    #debug concat(" -Placed ",str(_Load,0,0)," hairs, wrote ",str(_WrittenIHairs,0,0)," to disk.\n")
    #debug "Done!\n"
  #end
#end
